#include <sstream>
#include <cmath>
#include <cctype>

using namespace std;

#include "basfun.hh"
#include "error.hh"

long long int BasMODop(long long int a, long long int b)
{
    bool nega = a < 0;
    if(nega) a = -a;
    if(b < 0) b = -b;
    
    if(b == 0)
    {
        B_ERROR(11); // Division by zero
    }
    
    long long int result = a % b;
    return nega ? -result : result;
}

long long int BasCINT(long double a)
{
    if(a > 32767 || a < -32768) { B_ERROR(6); /* OVERFLOW */ }
    return (long long int)floorl(a + 0.5);
}
long long int BasCLNG(long double a)
{
    if(a >=  2147483647.5
    || a <  -2147483648.0) { B_ERROR(6); /* OVERFLOW */ }
    return (long long int)floorl(a + 0.5);
}

long long int BasINT(long double a)
{
    if(a > 32767 || a < -32768) { B_ERROR(6); /* OVERFLOW */ }
    return (long long int)(a);
}

long double BasVAL(const std::string& s)
{
    int base = 10;
    unsigned startpos = 0;
    if(s.size() >= 2 && s[0] == '&')
    {
        if(s[1] == 'H' || s[1] == 'h') { startpos = 2; base = 16; }
        if(s[1] == 'O' || s[1] == 'o') { startpos = 2; base = 8; }
        if(s[1] == 'B' || s[1] == 'b') { startpos = 2; base = 2; }
    }
    char* endpos;
    const char* beginpos = s.c_str() + startpos;
    long double result;
    if(base == 10)
        result = strtold(beginpos, &endpos);
    else
        result = strtoll(beginpos, &endpos, base);
   
    if(*endpos == '%')
    {
        for(; beginpos < endpos; ++beginpos)
            { if(*beginpos=='.' || (base==10 && std::toupper(*beginpos)=='E')) goto inv; }
        result = BasCINT(result);
    }
    else if(*endpos == '&')
    {
        for(; beginpos < endpos; ++beginpos)
            { if(*beginpos=='.' || (base==10 && std::toupper(*beginpos)=='E')) goto inv; }
        result = BasCLNG(result);
    }
    else if(*endpos == '!')
    {
        result = BasCSNG(result);
    }
    else if(*endpos == '#')
    {
        result = BasCDBL(result);
    }
    else if(*endpos)
    {
    inv:
        B_ERROR(5); // Invalid function call
    }
    
    return result;
}

const std::string BasLCASE(const std::string& s) { return LCase(s); }
const std::string BasUCASE(const std::string& s) { return UCase(s); }

const std::string BasSTRs(long double a)
{
    std::stringstream s;
    s << ' ' << a;
    return s.str();
}

const std::string BasCHRs(long long int a)
{
    if(a < 0 || a > 255)
    {
        B_ERROR(5); // Invalid function call
    }

    std::string s;
    s += (char)a;
    return s;
}


const std::string BasLEFTs(const std::string& s, int count)
{
    if(count < 0) B_ERROR(5);
    if((size_t)count >= s.size()) return s;
    return s.substr(0, count);
}

const std::string BasRIGHTs(const std::string& s, int count)
{
    if(count < 0) B_ERROR(5);
    if((size_t)count >= s.size()) return s;
    return s.substr(s.size()-count, count);
}

const std::string BasMIDs(const std::string& s, int pos, int count)
{
    if(pos <= 0 || count < 0) B_ERROR(5);
    if((size_t)pos > s.size()) return "";
    return s.substr(pos-1, count);
}
