template<typename T>
T& BAS_MAGICARRAY<T>::Index(unsigned numdims, ...)
{
    va_list ap;
    va_start(ap,numdims);
    T& res = impl->Index(numdims, ap);
    va_end(ap);
    return res;
}

template<typename T>
void BAS_MAGICARRAY<T>::Redim(unsigned numdims, ...)
{
    va_list ap;
    va_start(ap,numdims);
    impl->Redim(numdims, ap);
    va_end(ap);
}

template<typename T>
BAS_MAGICARRAY<T>::BAS_MAGICARRAY(T* data, unsigned numdims, ...)
    : impl(new BAS_MAGICARRAY_ImplSTATIC<T>(data, numdims))
{
    va_list ap;
    va_start(ap, numdims);
    
    BAS_MAGICARRAY_ImplSTATIC<T>* impl_st =
        static_cast < BAS_MAGICARRAY_ImplSTATIC<T>* > (&*impl);
    
    impl_st->InitDims(ap);
    va_end(ap);
}

template<typename T> template<unsigned numdims>
BAS_MAGICARRAY<T>::BAS_MAGICARRAY(BAS_DYNARRAY<T,numdims>& data)
    : impl(new BAS_MAGICARRAY_ImplDYNAMIC<T,numdims>(data))
{
}


template<typename T>
void BAS_MAGICARRAY_ImplSTATIC<T>::InitDims(va_list ap)
{
    for(unsigned a=0; a<dims.size(); ++a)
    {
        dims[a].first  = va_arg(ap, int);
        dims[a].second = va_arg(ap, unsigned) - dims[a].first + 1;
    }
}

template<typename T>
T& BAS_MAGICARRAY_ImplSTATIC<T>::Index(unsigned numdims, va_list ap)
{
    if(numdims != num_dims)
    {
     trouble:
        B_ERROR(9); /* subscript out of range */
    }
    unsigned res_index = 0;
    //unsigned res_mul   = 1;
    for(unsigned a=0; a<num_dims; ++a)
    {
        int req_index = va_arg(ap, int);
        if(req_index < dims[a].first) goto trouble;
        
        req_index -= dims[a].first;
        
        if(req_index >= (int)dims[a].second) goto trouble;
        
        //res_index += req_index * res_mul;
        //res_mul   *= dims[a].second;
        
        res_index *= dims[a].second;
        res_index += req_index;
    }
    
    //a + adim*(b + bdim*(c))
    
    //(((a)*bdim + b)*adim + a)
    
    return data[res_index];
}

template<typename T>
void BAS_MAGICARRAY_ImplSTATIC<T>::Erase()
{
    for(size_t pos=0, a=0; a<num_dims; ++a)
        for(size_t b=0; b<dims[a].second; ++b)
            data[pos++] = T();
}

template<typename T>
long BAS_MAGICARRAY_ImplSTATIC<T>::Lbound(unsigned dimindex) const
{
    if(dimindex > dims.size()) { B_ERROR(5); return -1; } // illegal function call
    if(dimindex > 0) --dimindex;
    return dims[dimindex].first;
}

template<typename T>
long BAS_MAGICARRAY_ImplSTATIC<T>::Ubound(unsigned dimindex) const
{
    if(dimindex > dims.size()) { B_ERROR(5); return -1; } // illegal function call
    if(dimindex > 0) --dimindex;
    return dims[dimindex].first + dims[dimindex].second-1;
}

template<typename T>
unsigned BAS_MAGICARRAY_ImplSTATIC<T>::Volume() const
{
    unsigned res = 1;
    for(unsigned a=0; a<num_dims; ++a)
        res *= dims[a].second;
    return res;
}
